@@ -149,16 +149,22 @@ def undo_args(args):
 
 
 def _pprint_displayhook(value):
-    if value is None or isinstance(value, HiddenCompletedCommand):
+    if value is None:
         return
     builtins._ = None  # Set '_' to None to avoid recursion
-    if HAS_PYGMENTS:
-        s = pretty(value)  # color case
-        lexer = pyghooks.XonshLexer()
-        tokens = list(pygments.lex(s, lexer=lexer))
+    if isinstance(value, HiddenCompletedCommand):
+        builtins._ = value
+        return
+    env = builtins.__xonsh_env__
+    if env.get('PRETTY_PRINT_RESULTS'):
+        printed_val = pretty(value)
+    else:
+        printed_val = repr(value)
+    if HAS_PYGMENTS and env.get('COLOR_RESULTS'):
+        tokens = list(pygments.lex(printed_val, lexer=pyghooks.XonshLexer()))
         print_color(tokens)
     else:
-        pprint(value)  # black & white case
+        print(printed_val)  # black & white case
     builtins._ = value
 
 
@@ -173,6 +179,7 @@ def premain(argv=None):
     """Setup for main xonsh entry point, returns parsed arguments."""
     if setproctitle is not None:
         setproctitle(' '.join(['xonsh'] + sys.argv[1:]))
+    builtins.__xonsh_ctx__ = {}
     args, other = parser.parse_known_args(argv)
     if args.file is not None:
         real_argv = (argv or sys.argv)
@@ -190,7 +197,8 @@ def premain(argv=None):
                     'completer': False,
                     'login': False,
                     'scriptcache': args.scriptcache,
-                    'cacheall': args.cacheall}
+                    'cacheall': args.cacheall,
+                    'ctx': builtins.__xonsh_ctx__}
     if args.login:
         shell_kwargs['login'] = True
     if args.config_path is None:
